//EStaticMesh.cpp
#include "EStaticMesh.h"
#include "../xmlnode/XmlNode.h"
#include "EDefine.h"
#include "XmlMesh.h"

EStaticMesh::EStaticMesh( LPDIRECT3DDEVICE9 pDevice ) : EMesh( pDevice )
{
}
//-----------------------------------------------------------------------------
EStaticMesh::~EStaticMesh()
{
}
//-----------------------------------------------------------------------------
const char* EStaticMesh::GetClassName()
{
	return "EStaticMesh";
}
//-----------------------------------------------------------------------------
int EStaticMesh::CreateVertexBuffer( XmlNode* pXmlNode )
{
	XmlMesh* pXmlMesh = NULL;
	if( strcmp( pXmlNode->GetClassName(), "XmlMesh" ) == 0 )
		pXmlMesh = (XmlMesh*)pXmlNode;

	unsigned	i;
	VOID*	pV;		//  lock  
	VOID*	pI;		// ε lock  
	std::vector<StaticVertex>	vtx;

	m_nVertex = (DWORD)pXmlMesh->m_vertex.size();
	m_dwFVF = StaticVertex::FVF;
	vtx.resize( m_nVertex );

	for( i = 0 ;  i < m_nVertex ; i++ )
	{
		vtx[i].p.x = pXmlMesh->m_vertex[i].p.x;
		vtx[i].p.y = pXmlMesh->m_vertex[i].p.y;
		vtx[i].p.z = pXmlMesh->m_vertex[i].p.z;
		vtx[i].n.x = pXmlMesh->m_vertex[i].n.x;
		vtx[i].n.y = pXmlMesh->m_vertex[i].n.y;
		vtx[i].n.z = pXmlMesh->m_vertex[i].n.z;
		vtx[i].t.x = pXmlMesh->m_vertex[i].t.x;
		vtx[i].t.y = pXmlMesh->m_vertex[i].t.y;
	}

	//   
	m_pDevice->CreateVertexBuffer( m_nVertex * sizeof(StaticVertex), 0, m_dwFVF, D3DPOOL_DEFAULT, &m_pVB, NULL );
	m_pVB->Lock( 0, m_nVertex * sizeof(StaticVertex), (void**)&pV, 0 );
	memcpy( pV, &vtx[0], m_nVertex * sizeof(StaticVertex) );
	m_pVB->Unlock();
	stl_wipe_vector( vtx );

	// ε 
	D3DCAPS9 caps;
	m_pDevice->GetDeviceCaps( &caps );
	m_nTriangle = (DWORD)pXmlMesh->m_index.size();

	// ε 32Ʈ ε ϸ 32Ʈ ε  
	if( caps.MaxVertexIndex > 0x0000ffff )
	{
		m_pDevice->CreateIndexBuffer( m_nTriangle * sizeof(base::Index3), 0, D3DFMT_INDEX32, D3DPOOL_DEFAULT, &m_pIB, NULL );
		m_pIB->Lock( 0, m_nTriangle * sizeof(base::Index3), (void**)&pI, 0 );
		DWORD* pW = (DWORD*)pI;
		for( i = 0 ; i < m_nTriangle ; i++ )
		{
			*(pW+0) = pXmlMesh->m_index[i].n[0];
			*(pW+1) = pXmlMesh->m_index[i].n[1];
			*(pW+2) = pXmlMesh->m_index[i].n[2];
			pW += 3;
		}
		m_pIB->Unlock();
	}
	else // ƴ϶ 16Ʈ ε ε  
	{
		m_pDevice->CreateIndexBuffer( m_nTriangle * sizeof(base::ushort3), 0, D3DFMT_INDEX16, D3DPOOL_DEFAULT, &m_pIB, NULL );
		m_pIB->Lock( 0, m_nTriangle * sizeof(base::ushort3), (void**)&pI, 0 );
		WORD* pW = (WORD*)pI;
		for( i = 0 ; i < m_nTriangle ; i++ )
		{
			*(pW+0) = (WORD)pXmlMesh->m_index[i].n[0];
			*(pW+1) = (WORD)pXmlMesh->m_index[i].n[1];
			*(pW+2) = (WORD)pXmlMesh->m_index[i].n[2];
			pW += 3;
		}
		m_pIB->Unlock();
	}

	return 1;
}
//-----------------------------------------------------------------------------
int EStaticMesh::OnDraw()
{
	m_pDevice->SetTransform( D3DTS_WORLD, &m_matWorld );
	m_pDevice->SetStreamSource( 0, m_pVB, 0, sizeof(StaticVertex) );
	m_pDevice->SetFVF( m_dwFVF );
	m_pDevice->SetIndices( m_pIB );
	m_pDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, 0, m_nVertex, 0, m_nTriangle );
	return 1;
}
//-----------------------------------------------------------------------------

