//XmlMesh.cpp
#include "XmlMesh.h"
#include "../xmlnode/XmlNode.h"
#include "../xmlnode/StringUtil.h"
#include "EDefine.h"
#include "EStaticMesh.h"

extern LPDIRECT3DDEVICE9   g_d3d_Device; 

XmlMesh::XmlMesh()
{
	m_bUseData = true;
	D3DXMatrixIdentity( &m_matLocal );
}
//-----------------------------------------------------------------------------
XmlMesh::~XmlMesh()
{
	m_vertex.clear();
	m_index.clear();
}
//-----------------------------------------------------------------------------
const char*	XmlMesh::GetClassName()
{
	return "XmlMesh";
}
//-----------------------------------------------------------------------------
void XmlMesh::SetUseData( bool bFlag )
{
	m_bUseData = bFlag;
}
//-----------------------------------------------------------------------------
bool XmlMesh::GetUseData()
{
	return m_bUseData;
}
//-----------------------------------------------------------------------------
void XmlMesh::SetUseDataChild( bool bFlag )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	for( ; iter != endIter; ++iter )
	{
		XmlMesh* pChild = (XmlMesh*)*iter;
		pChild->SetUseData( bFlag );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertMeshInfo()
{
	SetUseDataChild( false );
	XmlNode* pVertexXmlNode = FindFirstChild( "Vertex" );
	if( pVertexXmlNode == NULL )
		return;

	XmlMesh* pXmlMesh = (XmlMesh*)pVertexXmlNode;
	pXmlMesh->FetchVertex( m_vertex );
	
	XmlNode* pFaceXmlNode = FindFirstChild( "Face" );
	if( pFaceXmlNode  == NULL )
		return;
	
	pXmlMesh = (XmlMesh*)pFaceXmlNode;
	pXmlMesh->FetchFace( m_index );
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertTransformInfo()
{
	SetUseData( false );
	XmlNode* pLocalXmlNode = FindFirstChild( "LocalMatrix" );
	if( pLocalXmlNode == NULL )
		return;

	XmlMesh* pXmlMesh = (XmlMesh*)pLocalXmlNode;
	base::Matrix mat = ToMatrix( *pXmlMesh->GetData() );

	//"Transform" XML  Ŀ ȴ.
	if( strcmp( m_pParent->GetClassName(), "XmlMesh" ) == 0 )
	{
		XmlMesh* pParent = (XmlMesh*)m_pParent;
		ToDXMatrix( pParent->m_matLocal, mat ); 
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::ToDXMatrix( D3DXMATRIXA16& destMat, const base::Matrix& srcMat )
{
	destMat._11 = srcMat._11;	destMat._12 = srcMat._12;	destMat._13 = srcMat._13;	destMat._14 = srcMat._14;
	destMat._21 = srcMat._21;	destMat._22 = srcMat._22;	destMat._23 = srcMat._23;	destMat._24 = srcMat._24;
	destMat._31 = srcMat._31;	destMat._32 = srcMat._32;	destMat._33 = srcMat._33;	destMat._34 = srcMat._34;
	destMat._41 = srcMat._41;	destMat._42 = srcMat._42;	destMat._43 = srcMat._43;	destMat._44 = srcMat._44;
}
//-----------------------------------------------------------------------------
void XmlMesh::OnUpdate()
{
	if( *GetName() == "mesh" )
		ConvertMeshInfo(); 	
	else if( *GetName() == "Transform" )
		ConvertTransformInfo(); 
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchVertex( std::vector<VertexInfo>& vertexArr )
{
	//"Vertex" Ʈ - "Number" Ʈ ϱ
	XmlNode* pXmlNode = FindFirstChild( "Number" );
	if( pXmlNode == NULL )
		return;

	int number = ToInt( *pXmlNode->GetData() );
	if( number < 3 )
		return;

	//"Vertex" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for( int index = 0; iter != endIter; ++iter, ++index )
	{
		XmlNode* xmlNode = *iter;

		VertexInfo info;
		info.index = index;
		info.p = ToVector3( *xmlNode->GetData() );		
		vertexArr.push_back( info );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchFace( std::vector<base::Index3>& indexArr )
{
	//"Face" Ʈ - "Number" Ʈ ϱ
	XmlNode* pXmlNode = FindFirstChild( "Number" );
	if( pXmlNode == NULL )
		return;

	int number = ToInt( *pXmlNode->GetData() );
	if( number < 1 )
		return;

	//"Face" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for(; iter != endIter; ++iter )
	{
		XmlNode* xmlNode = *iter;
		base::Index3 index = ToIndex3( *xmlNode->GetData() );
		indexArr.push_back( index );
	}
}
//-----------------------------------------------------------------------------
ENode* XmlMesh::OnXmlToENode( ENode* pParent )
{
	ENode* pNode = NULL;
	if( m_vertex.size() > 0 )
	{
		EStaticMesh* pStaticMesh = new EStaticMesh( g_d3d_Device );
		pStaticMesh->CreateVertexBuffer( this );
		pNode = pStaticMesh;
	}
	else
	{
		pNode = new ENode( g_d3d_Device );
	}

	return pNode;
}
//-----------------------------------------------------------------------------
void XmlMesh::CreateMesh( ENode*& pParent )
{
	if( GetUseData() == false )
		return;

	ENode* pNode;
	if( pParent == NULL )
	{
		pNode = pParent = new ENode( g_d3d_Device );
	}
	else
	{
		pNode = OnXmlToENode( pParent );
		pParent->AttachChild( pNode );
	}

	pNode->SetMatrix( m_matLocal );
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	for( ; iter != endIter; ++iter )
	{
		XmlMesh* pChild = (XmlMesh*)*iter;
		pChild->CreateMesh( pNode );
	}	
}
